@@ -9,24 +9,23 @@ providing a module with two functions named ``integrate_adaptive`` and
 
 from __future__ import absolute_import, division, print_function
 
-
-from collections import defaultdict
 import copy
 import os
 import warnings
+from collections import defaultdict
 
 import numpy as np
 
-from .util import _ensure_4args, _default
 from .plotting import plot_result, plot_phase_plane
 from .results import Result
+from .util import _ensure_4args, _default
 
 
 class RecoverableError(Exception):
     pass
 
 
-class ODESys(object):
+class ODESys:
     """ Object representing an ODE system.
 
     ``ODESys`` provides unified interface to:
@@ -49,11 +48,20 @@ class ODESys(object):
         dependent variable (x). Signature is any of:
             - ``rhs(x, y[:]) -> f[:]``
             - ``rhs(x, y[:], p[:]) -> f[:]``
-            - ``rhs(x, y[:], p[:], backend=math) -> f[:]``
+            - ``rhs(x, y[:], p[:], backend=math) -> f[:]``.
     jac : callback
         Jacobian matrix (dfdy). Required for implicit methods.
+        Signature should be either of:
+            - ``jac(x, y[:]) -> J``
+            - ``jac(x, y[:], p[:]) -J``.
+        If ``nnz < 0``, ``J`` should be a 2D array-like object if ``nnz < 0``
+        corresponding to a dense or banded jacobian (see also ``band``).
+        If ``nnz >= 0``, ``J`` should be an instance of ``scipy.sparse.csc_matrix``.
     dfdx : callback
         Signature ``dfdx(x, y[:], p[:]) -> out[:]`` (used by e.g. GSL)
+    jtimes : callback
+        Jacobian-vector product (Jv). Signature is ```jtimes(x, y[:], v[:]) -> Jv[:]```
+        This is supported only by ``cvode``.
     first_step_cb : callback
         Signature ``step1st(x, y[:], p[:]) -> dx0`` (pass first_step==0 to use).
         This is available for ``cvode``, ``odeint`` & ``gsl``, but not for ``scipy``.
@@ -98,6 +106,9 @@ class ODESys(object):
         Describes whether the independent variable appears in the rhs expressions.
         If set to ``True`` the underlying solver is allowed to shift the
         independent variable during integration.
+    nnz : int (default : -1)
+        Maximum number of non-zero entries in sparse jacobian. When
+        non-negative, jacobian is assumed to be dense or banded.
 
     Attributes
     ----------
@@ -107,10 +118,13 @@ class ODESys(object):
         For evaluating the Jacobian matrix of f.
     dfdx_cb : callback or None
         For evaluating the second order derivatives.
+    jtimes_cb : callback or None
+        For evaluating the jacobian-vector product.
     first_step_cb : callback or None
         For calculating the first step based on x0, y0 & p.
     roots_cb : callback
     nroots : int
+    nnz : int
     names : tuple of strings
     param_names : tuple of strings
     description : str
@@ -142,14 +156,15 @@ class ODESys(object):
 
     """
 
-    def __init__(self, f, jac=None, dfdx=None, first_step_cb=None, roots_cb=None, nroots=None,
+    def __init__(self, f, jac=None, dfdx=None, jtimes=None, first_step_cb=None, roots_cb=None, nroots=None,
                  band=None, names=(), param_names=(), indep_name=None, description=None, dep_by_name=False,
                  par_by_name=False, latex_names=(), latex_param_names=(), latex_indep_name=None,
                  taken_names=None, pre_processors=None, post_processors=None, append_iv=False,
                  autonomous_interface=None, to_arrays_callbacks=None, autonomous_exprs=None,
-                 _indep_autonomous_key=None, numpy=None, **kwargs):
+                 _indep_autonomous_key=None, numpy=None, nnz=-1, **kwargs):
         self.f_cb = _ensure_4args(f)
         self.j_cb = _ensure_4args(jac) if jac is not None else None
+        self.jtimes_cb = _ensure_4args(jtimes) if jtimes is not None else None
         self.dfdx_cb = dfdx
         self.first_step_cb = first_step_cb
         self.roots_cb = roots_cb
@@ -158,6 +173,7 @@ class ODESys(object):
             if not band[0] >= 0 or not band[1] >= 0:
                 raise ValueError("bands needs to be > 0 if provided")
         self.band = band
+        self.nnz = nnz
         self.names = tuple(names or ())
         self.param_names = tuple(param_names or ())
         self.indep_name = indep_name
@@ -294,7 +310,7 @@ class ODESys(object):
             Final value of the independent variable.
         params : array_like
             See :meth:`integrate`.
-        \*\*kwargs :
+        \\*\\*kwargs :
             See :meth:`integrate`.
 
         Returns
@@ -316,7 +332,7 @@ class ODESys(object):
         xout : array_like
         params : array_like
             See :meth:`integrate`.
-        \*\*kwargs:
+        \\*\\*kwargs:
             See :meth:`integrate`
 
         Returns
@@ -363,6 +379,11 @@ class ODESys(object):
             Whether to use the jacobian. When ``None`` the choice is
             done automatically (only used when required). This matters
             when jacobian is derived at runtime (high computational cost).
+        with_jtimes : bool (default: False)
+            Whether to use the jacobian-vector product. This is only supported
+            by ``cvode`` and only when ``linear_solver`` is one of: gmres',
+            'gmres_classic', 'bicgstab', 'tfqmr'. See the documentation
+            for ``pycvodes`` for more information.
         force_predefined : bool (default: False)
             override behaviour of ``len(x) == 2`` => :meth:`adaptive`
         \\*\\*kwargs :
@@ -440,11 +461,11 @@ class ODESys(object):
 
         Parameters
         ----------
-        \*args :
+        \\*args :
             See :meth:`integrate`.
         name : str (default: 'lsoda'/'dopri5' when jacobian is available/not)
             What integrator wrapped in scipy.integrate.ode to use.
-        \*\*kwargs :
+        \\*\\*kwargs :
             Keyword arguments passed onto `set_integrator(...) <
         http://docs.scipy.org/doc/scipy/reference/generated/
         scipy.integrate.ode.set_integrator.html#scipy.integrate.ode.set_integrator>`_
@@ -565,21 +586,44 @@ class ODESys(object):
                     return 1  # recoverable error
 
             if with_jacobian is None:
-                raise ValueError("Need to pass with_jacobian")
+                raise Exception("Must pass with_jacobian")
             elif with_jacobian is True:
-                def _j(x, y, jout, dfdx_out=None, fy=None):
-                    if len(_p) > 0:
-                        jout[:, :] = np.asarray(self.j_cb(x, y, _p))
-                    else:
-                        jout[:, :] = np.asarray(self.j_cb(x, y))
-                    if dfdx_out is not None:
+                if self.nnz >= 0:
+                    def _j(x, y, data, colptrs, rowvals):
                         if len(_p) > 0:
-                            dfdx_out[:] = np.asarray(self.dfdx_cb(x, y, _p))
+                            J = self.j_cb(x, y, _p)
                         else:
-                            dfdx_out[:] = np.asarray(self.dfdx_cb(x, y))
+                            J = self.j_cb(x, y)
+                        J = J.asformat("csc")
+                        data[:] = J.data
+                        colptrs[:] = J.indptr
+                        rowvals[:] = J.indices
+
+                    new_kwargs['nnz'] = self.nnz
+                else:
+                    def _j(x, y, jout, dfdx_out=None, fy=None):
+                        if len(_p) > 0:
+                            jout[:, :] = np.asarray(self.j_cb(x, y, _p))
+                        else:
+                            jout[:, :] = np.asarray(self.j_cb(x, y))
+                        if dfdx_out is not None:
+                            if len(_p) > 0:
+                                dfdx_out[:] = np.asarray(self.dfdx_cb(x, y, _p))
+                            else:
+                                dfdx_out[:] = np.asarray(self.dfdx_cb(x, y))
             else:
                 _j = None
 
+            with_jtimes = kwargs.pop('with_jtimes', False)
+            if with_jtimes is True:
+                def _jtimes(v, Jv, x, y, fy=None):
+                    yv = np.concatenate((y, v))
+                    if len(_p) > 0:
+                        Jv[:] = np.asarray(self.jtimes_cb(x, yv, _p))
+                    else:
+                        Jv[:] = np.asarray(self.jtimes_cb(x, yv))
+                new_kwargs['jtimes'] = _jtimes
+
             if self.first_step_cb is not None:
                 def _first_step(x, y):
                     if len(_p) > 0:
@@ -604,6 +648,7 @@ class ODESys(object):
                     if 'nroots' in new_kwargs:
                         raise ValueError("cannot override nroots")
                     new_kwargs['nroots'] = self.nroots
+
             if nx == 2 and not force_predefined:
                 _xout, yout, info = adaptive(_f, _j, _y0, *_xout, **new_kwargs)
                 info['mode'] = 'adaptive'
@@ -626,11 +671,11 @@ class ODESys(object):
 
         Parameters
         ----------
-        \*args :
+        \\*args :
             see :meth:`integrate`
         method : str (default: 'bsimp')
             what stepper to use, see :py:attr:`gslodeiv2.steppers`
-        \*\*kwargs :
+        \\*\\*kwargs :
             keyword arguments passed onto
             :py:func:`gslodeiv2.integrate_adaptive`/:py:func:`gslodeiv2.integrate_predefined`
 
@@ -667,14 +712,14 @@ class ODESys(object):
         (via `pycvodes <https://pypi.python.org/pypi/pycvodes>`_)
         to integrate the ODE system. """
         import pycvodes  # Python interface to SUNDIALS's cvodes integrators
-        kwargs['with_jacobian'] = kwargs.get(
-            'method', 'bdf') in pycvodes.requires_jac
+        kwargs['with_jacobian'] = kwargs.get('method', 'bdf') in pycvodes.requires_jac
         if 'lband' in kwargs or 'uband' in kwargs or 'band' in kwargs:
             raise ValueError("lband and uband set locally (set at"
                              " initialization instead)")
         if self.band is not None:
             kwargs['lband'], kwargs['uband'] = self.band
         kwargs['autonomous_exprs'] = self.autonomous_exprs
+
         return self._integrate(pycvodes.integrate_adaptive,
                                pycvodes.integrate_predefined,
                                *args, **kwargs)
@@ -789,7 +834,7 @@ def integrate_auto_switch(odes, kw, x, y0, params=(), **kwargs):
     x : array_like
     y0 : array_like
     params : array_like
-    \*\*kwargs:
+    \\*\\*kwargs:
         See :meth:`ODESys.integrate`
 
     Notes
