@@ -15,6 +15,12 @@ def _set_scale(cb, argstr):
         cb(arg, **eval('dict(%s)' % kw))
 
 
+def _latex_from_dimensionality(dim):
+    # see https://github.com/python-quantities/python-quantities/issues/148
+    from quantities.markup import format_units_latex
+    return format_units_latex(dim, mult=r'\\cdot')
+
+
 def plot_result(x, y, indices=None, plot_kwargs_cb=None, ax=None,
                 ls=('-', '--', ':', '-.'),
                 c=('tab:blue', 'tab:orange', 'tab:green', 'tab:red', 'tab:purple',
@@ -22,7 +28,8 @@ def plot_result(x, y, indices=None, plot_kwargs_cb=None, ax=None,
                 m=('o', 'v', '8', 's', 'p', 'x', '+', 'd', 's'),
                 m_lim=-1, lines=None, interpolate=None, interp_from_deriv=None,
                 names=None, latex_names=None, xlabel=None, ylabel=None,
-                xscale=None, yscale=None, legend=False, yerr=None, labels=None):
+                xscale=None, yscale=None, legend=False, yerr=None, labels=None, tex_lbl_fmt='$%s$',
+                fig_kw=None, xlim=None, ylim=None):
     """
     Plot the depepndent variables vs. the independent variable
 
@@ -70,7 +77,7 @@ def plot_result(x, y, indices=None, plot_kwargs_cb=None, ax=None,
     import matplotlib.pyplot as plt
 
     if ax is None:
-        ax = plt.subplot(1, 1, 1)
+        _fig, ax = plt.subplots(1, 1, **(fig_kw or {}))
     if plot_kwargs_cb is None:
         def plot_kwargs_cb(idx, lines=False, markers=False, labels=None):
 
@@ -111,14 +118,14 @@ def plot_result(x, y, indices=None, plot_kwargs_cb=None, ax=None,
             clr = plot_kwargs_cb(idx)['c']
             ax.fill_between(x, _y[:, idx] - yerr[:, idx], _y[:, idx] + yerr[:, idx], facecolor=clr, alpha=.3)
 
-    if isinstance(yscale, str) and 'linthreshy' in yscale:
+    if isinstance(yscale, str) and 'linthresh' in yscale:
         arg, kw = yscale.split(';')
-        thresh = eval('dict(%s)' % kw)['linthreshy']
+        thresh = eval('dict(%s)' % kw)['linthresh']
         ax.axhline(thresh, linewidth=.5, linestyle='--', color='k', alpha=.5)
         ax.axhline(-thresh, linewidth=.5, linestyle='--', color='k', alpha=.5)
 
     if labels is None:
-        labels = names if latex_names is None else ['$%s$' % ln.strip('$') for ln in latex_names]
+        labels = names if latex_names is None else [tex_lbl_fmt % ln.strip('$') for ln in latex_names]
 
     for idx in indices:
         ax.plot(x, _y[:, idx], **plot_kwargs_cb(
@@ -129,7 +136,7 @@ def plot_result(x, y, indices=None, plot_kwargs_cb=None, ax=None,
 
     if xlabel is None:
         try:
-            ax.set_xlabel(x.dimensionality.latex)
+            ax.set_xlabel(_latex_from_dimensionality(x.dimensionality))
         except AttributeError:
             pass
     else:
@@ -137,7 +144,7 @@ def plot_result(x, y, indices=None, plot_kwargs_cb=None, ax=None,
 
     if ylabel is None:
         try:
-            ax.set_ylabel(_y.dimensionality.latex)
+            ax.set_ylabel(_latex_from_dimensionality(_y.dimensionality))
         except AttributeError:
             pass
     else:
@@ -185,6 +192,11 @@ def plot_result(x, y, indices=None, plot_kwargs_cb=None, ax=None,
         pass
     else:
         ax.legend(**legend)
+
+    if xlim:
+        ax.set_xlim(xlim)
+    if ylim:
+        ax.set_ylim(ylim)
     return ax
 
 
@@ -203,7 +215,7 @@ def plot_phase_plane(x, y, indices=None, plot=None, names=None, ax=None, **kwarg
         Uses ``matplotlib.pyplot.plot`` if ``None``
     names: iterable of strings
         Labels for x and y axis.
-    \*\*kwargs:
+    \\*\\*kwargs:
         Keyword arguemtns passed to ``plot()``.
 
     """
