@@ -7,9 +7,10 @@ from .plotting import plot_result, plot_phase_plane, info_vlines
 from .util import import_
 
 CubicSpline = import_('scipy.interpolate', 'CubicSpline')
+interp1d = import_('scipy.interpolate', 'interp1d')
 
 
-class Result(object):
+class Result:
 
     def __init__(self, xout, yout, params, info, odesys):
         self.xout = xout
@@ -17,6 +18,7 @@ class Result(object):
         self.params = params
         self.info = info
         self.odesys = odesys
+        self.names = self.odesys.names
 
     def copy(self):
         return Result(self.xout.copy(), self.yout.copy(), self.params.copy(),
@@ -54,8 +56,28 @@ class Result(object):
         select_l = xtmp > lower
         return xtmp[..., select_l], ytmp[..., select_l, :]
 
-    def at(self, x, use_deriv=False, xdata=None, ydata=None):
-        """ Returns interpolated result at a given time and an interpolation error-estimate """
+    def at(self, x, use_deriv=False, xdata=None, ydata=None, linear=False):
+        """ Returns interpolated result at a given time and an interpolation error-estimate
+
+        By default interpolation is performed using cubic splines.
+
+        Parameters
+        ----------
+        x : array_like or float
+        use_deriv : bool
+            Calculate derivatives at spline knots for enhanced accuracy.
+        xdata : array
+        ydata : array
+        linear : bool
+            Will use (cheaper) linear interpolation. Useful when x is an array.
+            Error estimate will be ``None`` in this case.
+
+        Returns
+        -------
+        interpolated_y : array
+        error_estim_y : array
+
+        """
         if xdata is None:
             xdata = self.xout
         if ydata is None:
@@ -63,6 +85,16 @@ class Result(object):
         yunit = getattr(ydata, 'units', 1)
         ydata = getattr(ydata, 'magnitude', ydata)
 
+        if linear:
+            return interp1d(xdata, ydata, axis=0)(x)*yunit, None
+        else:
+            try:
+                len(x)
+            except TypeError:
+                pass
+            else:
+                return [self.at(_, use_deriv, xdata, ydata, linear) for _ in x]
+
         if x == xdata[0]:
             res = ydata[0, :]
             err = res*0
@@ -182,7 +214,7 @@ class Result(object):
         deriv : bool
             Plot derivatives (internal variables).
         names : iterable of str
-        \*\*kwargs:
+        \\*\\*kwargs:
             See :func:`pyodesys.plotting.plot_result`
         """
         if between is not None:
@@ -224,7 +256,7 @@ class Result(object):
         ----------
         indices : iterable of int
         names : iterable of str
-        \*\*kwargs:
+        \\*\\*kwargs:
             See :func:`pyodesys.plotting.plot_phase_plane`
 
         """
@@ -254,7 +286,7 @@ class Result(object):
                 else self.odesys.numpy.linspace(x0, xend, npoints+1)
             ), self.yout[..., -1, :], params or self.params, **kwargs
         )
-        self.xout = self.odesys.numpy.concatenate((self.xout, res.xout[1:] + (x0 if autonomous else 0)))
+        self.xout = self.odesys.numpy.concatenate((self.xout, res.xout[1:] + (x0 if autonomous else 0*x0)))
         self.yout = self.odesys.numpy.concatenate((self.yout, res.yout[..., 1:, :]))
         new_info = {k: v for k, v in self.info.items() if not (
             k.startswith('internal') and odesys is not self.odesys)}
@@ -267,7 +299,9 @@ class Result(object):
             elif k == 'success':
                 new_info[k] = new_info[k] and v
             elif k.endswith('_xvals'):
-                new_info[k] = self.odesys.numpy.concatenate((new_info[k], v + (x0 if autonomous else 0)))
+                if len(v) == 0:
+                    continue
+                new_info[k] = self.odesys.numpy.concatenate((new_info[k], v + (x0 if autonomous else 0*x0)))
             elif k.endswith('_indices'):
                 new_info[k].extend([itm + nx0 - 1 for itm in v])
             elif isinstance(v, str):
