@@ -5,10 +5,16 @@ from functools import reduce
 import inspect
 import math
 import operator
+import sys
 
 from pkg_resources import parse_requirements, parse_version
 
 import numpy as np
+import pytest
+
+if sys.version_info < (3, 6, 0):
+    class ModuleNotFoundError(ImportError):
+        pass
 
 
 def stack_1d_on_left(x, y):
@@ -107,7 +113,7 @@ def transform_exprs_indep(fw, bw, dep_exprs, indep, check=True):
     return [(e/fw.diff(indep)).subs(indep, bw) for e in exprs]
 
 
-class _Blessed(object):
+class _Blessed:
     pass
 
 
@@ -130,11 +136,15 @@ def _ensure_4args(func):
         return func
 
     self_arg = 1 if inspect.ismethod(func) else 0
-    if len(inspect.getargspec(func)[0]) == 4 + self_arg:
+    if hasattr(inspect, 'getfullargspec'):
+        args = inspect.getfullargspec(func)[0]
+    else:  # Python 2:
+        args = inspect.getargspec(func)[0]
+    if len(args) == 4 + self_arg:
         return func
-    if len(inspect.getargspec(func)[0]) == 3 + self_arg:
+    if len(args) == 3 + self_arg:
         return lambda x, y, p=(), backend=math: func(x, y, p)
-    elif len(inspect.getargspec(func)[0]) == 2 + self_arg:
+    elif len(args) == 2 + self_arg:
         return lambda x, y, p=(), backend=math: func(x, y)
     else:
         raise ValueError("Incorrect numer of arguments")
@@ -180,7 +190,7 @@ class _Callback(_Blessed):
         return result
 
 
-class requires(object):
+class requires:
     """ Conditional skipping (on requirements) of tests in pytest
 
     Examples
@@ -225,7 +235,6 @@ class requires(object):
                             self.incomp.append(str(req))
 
     def __call__(self, cb):
-        import pytest
         r = 'Unfulfilled requirements.'
         if self.missing:
             r += " Missing modules: %s." % ', '.join(self.missing)
@@ -234,7 +243,27 @@ class requires(object):
         return pytest.mark.skipif(self.missing or self.incomp, reason=r)(cb)
 
 
-class MissingImport(object):
+def pycvodes_double(cb):
+    try:
+        from pycvodes import config
+        prec = config.get('SUNDIALS_PRECISION', 'double')
+    except:
+        prec = "double"
+    r = "Test is designed only for pycvodes built with double precision."
+    return pytest.mark.skipif(prec != "double", reason=r)(cb)
+
+
+def pycvodes_klu(cb):
+    try:
+        from pycvodes import config
+        klu = config['KLU']
+    except (ModuleNotFoundError, ImportError):
+        klu = False
+    return pytest.mark.skipif(not klu,
+                              reason="Sparse jacobian tests require pycvodes and sundials with KLU enabled.")(cb)
+
+
+class MissingImport:
 
     def __init__(self, modname, exc):
         self._modname = modname
@@ -246,6 +275,9 @@ class MissingImport(object):
         else:
             raise self._exc  # ImportError("Failed to import %s" % self._modname)
 
+    def __getitem__(self, key):
+        raise self._exc  # ImportError("Failed to import %s" % self._modname)
+
     def __call__(self, *args, **kwargs):
         raise self._exc  # ImportError("Failed to import %s" % self._modname)
 
